/*
 * Author:          Patrick-Christopher Mattulat
 * Company:         Lynar Studios
 * E-Mail:          webmaster@lynarstudios.com
 * Created:         2020-08-20
 * Changed:         2023-05-19
 *
 * */

#ifndef LS_STD_LOG_LEVEL_HPP
#define LS_STD_LOG_LEVEL_HPP

#include "LogLevelValue.hpp"
#include <ls-std/core/Class.hpp>
#include <ls-std/os/dynamic-goal.hpp>
#include <string_view>
#include <unordered_map>

namespace ls::std::io
{
  class LS_STD_DYNAMIC_GOAL LogLevel : public ls::std::core::Class
  {
    public:

      explicit LogLevel(const ls::std::io::LogLevelValue &_value);
      LogLevel();
      ~LogLevel() noexcept override;

      ls::std::io::LogLevel &operator=(const ls::std::io::LogLevelValue &_value);
      bool operator<(const ls::std::io::LogLevelValue &_value) const;
      bool operator<=(const ls::std::io::LogLevelValue &_value) const;
      bool operator>(const ls::std::io::LogLevelValue &_value) const;
      bool operator>=(const ls::std::io::LogLevelValue &_value) const;
      bool operator==(const ls::std::io::LogLevelValue &_value) const;

      [[nodiscard]] ls::std::io::LogLevelValue getValue() const;
      void setLogLevel(const ls::std::io::LogLevelValue &_value);
      void setLogLevel(const ::std::string &_value);
      [[nodiscard]] ::std::string toString() const;

    private:

      ::std::unordered_map<ls::std::io::LogLevelValue, ::std::string> level{};
      ls::std::io::LogLevelValue value{};

      [[nodiscard]] ls::std::io::LogLevelValue _getValueFromString(const ::std::string &_value);
      void _init();
      [[nodiscard]] bool _isValidLogLevelString(::std::string_view _value) const;
  };
}

#endif
